/* eslint-env node */
'use strict';

const fastboot = require('fastboot-transform');

const preset = {
	modes: [],
	themes: [],
	codemirror: false,
	includeTags: false,
	includeComments: false,
	includeBrackets: false,
	includeWhitespace: false,
};

module.exports = {
	name: 'ember-editables',
	included(app) {

		this._super.included(app);

		let config = this.project.config(process.env.EMBER_ENV).editables || {};

		Object.keys(preset).forEach(function(key) {
			if (typeof config[key] === 'undefined') {
				config[key] = preset[key];
			}
		});

		if (config.codemirror) {

			app.import('vendor/codemirror/lib/codemirror.js');
			app.import('vendor/codemirror/lib/codemirror.css');
			app.import('vendor/codemirror/addon/mode/simple.js');
			app.import('vendor/codemirror/addon/mode/multiplex.js');

			if (config.includeTags) {
				app.import('vendor/codemirror/addon/edit/closetag.js');
				app.import('vendor/codemirror/addon/edit/matchtags.js');
			}

			if (config.includeBrackets) {
				app.import('vendor/codemirror/addon/edit/closebrackets.js');
				app.import('vendor/codemirror/addon/edit/matchbrackets.js');
			}

			if (config.includeComments) {
				app.import('vendor/codemirror/addon/comment/comment.js');
			}

			if (config.includeWhitespace) {
				app.import('vendor/codemirror/addon/edit/trailingspace.js');
			}

			config.modes.forEach(function(m) {
				app.import('vendor/codemirror/mode/'+m+'/'+m+'.js');
			});

			config.themes.forEach(function(t) {
				app.import('vendor/codemirror/theme/'+t+'.css');
			});

			app.import('vendor/codemirror.css');
			app.import('vendor/codemirror.js', {
				exports: {
					CodeMirror: ['default']
				}
			});

		}

	},

	options: {
    nodeAssets: {
      codemirror() {

      	var imports = [];
      	var vendors = [];

      	var config = this.project.config(process.env.EMBER_ENV).editables || {};

      	Object.keys(preset).forEach(function(key) {
      		if (typeof config[key] === 'undefined') {
      			config[key] = preset[key];
      		}
      	});

      	if (config.codemirror) {

      		vendors.push('lib/codemirror.js');
      		imports.push('lib/codemirror.css');
      		vendors.push('addon/mode/simple.js');
      		vendors.push('addon/mode/multiplex.js');

      		if (config.includeTags) {
      			vendors.push('addon/edit/closetag.js');
      			vendors.push('addon/edit/matchtags.js');
      		}

      		if (config.includeBrackets) {
      			vendors.push('addon/edit/closebrackets.js');
      			vendors.push('addon/edit/matchbrackets.js');
      		}

      		if (config.includeComments) {
      			vendors.push('addon/comment/comment.js');
      		}

      		if (config.includeWhitespace) {
      			vendors.push('addon/edit/trailingspace.js');
      		}

      		config.modes.forEach(function(m) {
      			vendors.push('mode/'+m+'/'+m+'.js');
      		});

      		config.themes.forEach(function(t) {
      			imports.push('theme/'+t+'.css');
      		});

      	}

      	return {
          import: imports,
          vendor: {
          	include: vendors,
	          processTree(input) {
	            return fastboot(input);
	          },
	        },
        };

      }
    }
  },

};
